/*****************************************************************************
 * Author:   Valient Gough <vgough@pobox.com>
 *
 *****************************************************************************
 * Copyright (c) 2002-2003, Valient Gough
 *
 * This library is free software; you can distribute it and/or modify it under
 * the terms of the GNU General Public License (GPL), as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GPL in the file COPYING for more
 * details.
 *
 */

#ifndef _Cypher_incl_
#define _Cypher_incl_

class Cypher
{
public:
    Cypher();
    ~Cypher();

    // create a new key based on a password
    void *newKey(const char *password, int passwdLength) ;
    // create a new random key
    void *newRandomKey() ;

    void destroyKey(void *key) ;

    // data must be len keySize()
    void *readKey(const unsigned char *data, void *encodingKey) ; 
    void writeKey(const void *key, unsigned char *data, void *encodingKey) ; 

    // meta-data about the cypher
    int keySize() const;
    int blockSize() const;

    // returns size of encoded name based on input length.
    int encodedNameLen(int plaintextNameLen) const;
    int decodedNameLen(int encodedNameLen) const;

    // functional interfaces
    /*
	Name encoding can have a different output length from input length and
	is useful mostly for encoding file names.  Use encodedNameLen() to find
	output length of encodeName.
    */
    void encodeName(const char *in, int len, char *out, void *key);
    bool decodeName(const char *in, int len, char *out, void *key);

    /*
	Block encoding is done in-place.  Partial blocks are supported, but
	blocks are always expected to begin on a block boundary.  See
	blockSize().
    */
    bool encodeBlock(char *buf, int size, unsigned long blockNum, void *key);
    bool decodeBlock(char *buf, int size, unsigned long blockNum, void *key);
};


#endif

