/*****************************************************************************
 * Author:   Valient Gough <vgough@pobox.com>
 *
 *****************************************************************************
 * Copyright (c) 2003, Valient Gough
 *
 * This library is free software; you can distribute it and/or modify it under
 * the terms of the GNU General Public License (GPL), as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GPL in the file COPYING for more
 * details.
 *
 */

#include "MemoryPool.h"
#include "rlog.h"

#include <stdlib.h>
#include <string.h>

struct BlockList
{
    BlockList *next;
    char el[1];
};

MemoryPool::MemoryPool( int elementSize )
    : available(0)
    , elSize( elementSize )
    , numBlocks( 0 )
{
    BlockList tmp;
    offset = (char*)&tmp.el[0] - (char*)&tmp;
}

MemoryPool::~MemoryPool()
{
    // Don't free the blocks in the destructor, because we wan't to leave them
    // on program exit so that a leak detector can see how many blocks were in
    // use..
}

void MemoryPool::freeAvailable()
{
    while(available)
    {
	BlockList *next = available->next;
	free( (void*)available );
	available = next;
    }
}

void *MemoryPool::allocate()
{
    if(!available)
    {
	BlockList *block = (BlockList*)malloc( sizeof(BlockList) + elSize );
	block->next = available;
	available = block;
	++numBlocks;
	//rDebug("allocated new block, up to %i", numBlocks);
    }
    BlockList *block = available;
    available = block->next;
    block->next = 0;
    return &block->el[0];
}

void MemoryPool::release(void *el)
{
    BlockList *block = (BlockList*)((char*)el - offset);

    if(block->next != 0 || block->el != el)
    {
	rError("Attempting to release an invalid pointer!");
	return;
    }

    memset(el, 0, elSize);
    block->next = available;
    available = block;
}

