/*****************************************************************************
 * Author:   Valient Gough <vgough@pobox.com>
 *
 *****************************************************************************
 * Copyright (c) 2002-2003, Valient Gough
 *
 * This library is free software; you can distribute it and/or modify it under
 * the terms of the GNU General Public License (GPL), as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GPL in the file COPYING for more
 * details.
 *
 */

#include "rlog.h"

#include <stdio.h>
#include <stdarg.h>

#define UNUSED(x) (void)(x)

using namespace std;

const int SLogMajor = 1;
const int SLogMinor = 0;

static bool gEnabled = true;

void RLog_Disable()
{
    gEnabled = false;
}

struct RLogNode
{
    RLogType type;
};

RLogNode GlobalDebugNode = { RLog_Debug };
RLogNode GlobalWarningNode = { RLog_Warning };
RLogNode GlobalErrorNode = { RLog_Error };

/*
    Vast simplification, we don't do any real tracking yet or allow
    enabling/disabling of messages, or even allow messages to be caught by
    other handlers!

    There are other versions of this logging code around which have more
    features.  This is a minimal implementation placeholder for now.
*/
RLogPublisher *RLog_Register(struct RLogPublisher **, enum RLogType type,
	const char *fileName, const char *functionName, int lineNum)
{
    RLogPublisher *pub = new RLogPublisher;
    switch(type)
    {
    case RLog_Debug:
	pub->node = &GlobalDebugNode;
	break;
    case RLog_Warning:
	pub->node = &GlobalWarningNode;
	break;
    case RLog_Error:
	pub->node = &GlobalErrorNode;
	break;
    }

    pub->lineNum = lineNum;
    pub->fileName = fileName;
    pub->functionName = functionName;

    return pub;
}

void RLog_Publish( struct RLogPublisher **pub, const char *format, ... )
{
    if(!gEnabled) // TODO: to be removed when rlog gets update
	return;

    switch((*pub)->node->type)
    {
    case RLog_Debug:
	fprintf(stderr, "Debug: ");
	break;
    case RLog_Warning:
	fprintf(stderr, "Warning: ");
	break;
    case RLog_Error:
	fprintf(stderr, "Error: ");
	break;
    }

    fprintf(stderr, "%s:%i ", (*pub)->fileName, (*pub)->lineNum);

    va_list ap;
    va_start(ap, format);

    vfprintf(stderr, format, ap);

    va_end(ap);

    fprintf(stderr, "\n");

    return;
}


