/*****************************************************************************
 * Author:   Valient Gough <vgough@pobox.com>
 *
 *****************************************************************************
 * Copyright (c) 2003, Valient Gough
 *
 * This library is free software; you can distribute it and/or modify it under
 * the terms of the GNU General Public License (GPL), as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GPL in the file COPYING for more
 * details.
 *
 */

#include <iostream>

#include <stdlib.h>

#include <openssl/ssl.h>

#include "Cypher.h"
#include "FileInfo.h"


using namespace std;

Cypher *gCypher = 0;
void *gVolumeKey = 0;

int checkErrorPropogation( int byteToChange, void *key )
{
    const int DataSize = 63;
    char data[DataSize];

    for(int i=0; i<DataSize; ++i)
	data[i] = i;

    gCypher->encodeBlock( data, DataSize, 0, key );
   
    // intoduce an error in the encoded data, so we can check error propogation
    if(byteToChange >= 0 && byteToChange < DataSize)
	--data[byteToChange];

    gCypher->decodeBlock( data, DataSize, 0, key );

    int numByteErrors = 0;
    for(int i=0; i<DataSize; ++i)
    {
	if( data[i] != i )
	    ++numByteErrors;
    }

    return numByteErrors;
}

int main(int argc, char *argv[])
{
    SSL_load_error_strings();
    SSL_library_init();

    gCypher = new Cypher();

    // create a random key
    void *key = gCypher->newRandomKey();

    // encrypt a name
    const char *name[] = {
	"test-name",
	"test-name2",
	"test-name3",
	"../test",
	"test-name.21",
	"test-name.22",
	"test-name.23",
	"test-name.o",
	0 
    };

    cerr << "Testing name encode/decode\n";

    const char **orig = name;
    while(*orig)
    {
	cerr << "testing encoding of name \"" << *orig << "\"";

	string encName = FileInfo::encodeName( *orig, gCypher, key );

	cerr << " -> \"" << encName.c_str() << "\"";

	// decrypt name
	string decName = FileInfo::decodeName( encName.c_str(), 
		gCypher, key );

	if(decName == *orig)
	    cerr << "   OK\n";
	else
	{
	    cerr << "   FAILED!\n";
	    return 1;
	}

	orig++;
    }
    
    cerr << "Testing block encode/decode -  ";
    {
	int numErrors = checkErrorPropogation( -1, key );
	if(numErrors)
	{
	    cerr << " FAILED!\n";
	    return 1;
	} else
	    cerr << " OK\n";
    }

    cerr << "Checking error propogation:\n";
    {
	for(int i=0; i<32; ++i)
	{
	    int numErrors = checkErrorPropogation( i, key );

	    cerr << "Modification of byte " << i << " affected " 
		<< numErrors << " decoded bytes\n";
	}
    }



    gCypher->destroyKey( key );
    delete gCypher;

    return 0;
}



